<?php
/**
 * Admin Settings Pages Helper.
 *
 * @package    BuddyPress_Auto_Friendship_Pro
 * @subpackage Admin
 * @copyright  Copyright (c) 2021, Brajesh Singh
 * @license    https://www.gnu.org/licenses/gpl.html GNU Public License
 * @author     Ravi Sharma, Brajesh Singh
 * @since      1.0.0
 */

namespace BuddyPress_Auto_Friendship_Pro\Admin;

use \Press_Themes\PT_Settings\Page;

// Do not allow direct access over web.
defined( 'ABSPATH' ) || exit;

/**
 * Class Admin_Settings
 */
class Admin_Settings {

	/**
	 * Admin Menu slug
	 *
	 * @var string
	 */
	private $menu_slug;

	/**
	 * Used to keep a reference of the Page, It will be used in rendering the view.
	 *
	 * @var \Press_Themes\PT_Settings\Page
	 */
	private $page;

	/**
	 * Boot settings
	 */
	public static function boot() {
		$self = new self();
		$self->setup();
	}

	/**
	 * Setup settings
	 */
	public function setup() {
		$this->menu_slug = 'bp-auto-friendship-settings';

		add_action( 'admin_init', array( $this, 'init' ) );
		add_action( 'admin_menu', array( $this, 'add_menu' ) );
	}

	/**
	 * Show/render the setting page
	 */
	public function render() {
		$this->page->render();
	}

	/**
	 * Is it the setting page?
	 *
	 * @return bool
	 */
	private function needs_loading() {

		global $pagenow;

		// We need to load on options.php otherwise settings won't be reistered.
		if ( 'options.php' === $pagenow ) {
			return true;
		}

		if ( isset( $_GET['page'] ) && $_GET['page'] === $this->menu_slug ) {
			return true;
		}

		return false;
	}

	/**
	 * Initialize the admin settings panel and fields
	 */
	public function init() {

		if ( ! $this->needs_loading() ) {
			return;
		}

		$page = new Page( 'bp_auto_friendship_pro', __( 'Auto Friendship Pro Settings', 'buddypress-auto-friendship-pro' ) );

		// General settings tab.
		$general = $page->add_panel( 'general', _x( 'General', 'Admin settings panel title', 'buddypress-auto-friendship-pro' ) );

		$section_general = $general->add_section( 'settings', _x( 'General Settings', 'Admin settings section title', 'buddypress-auto-friendship-pro' ) );

		$defaults = bpafp_get_default_options();

		$fields = array(
			array(
				'name'    => 'auto_friendship_triggers',
				'label'   => _x( 'Auto Friendship on', 'Admin settings', 'buddypress-auto-friendship-pro' ),
				'desc'    => __( 'On selected actions Auto Friendship will be created for user', 'buddypress-auto-friendship-pro' ),
				'type'    => 'multicheck',
				'options' => bpafp_get_auto_friendship_triggers(),
				'default' => $defaults['auto_friendship_triggers'],
			),
		);

		$section_general->add_fields( $fields );

		do_action( 'bpafp_admin_settings_page', $page );

		// General settings tab.
		$misc_panel = $page->add_panel( 'misc', _x( 'Misc', 'Admin settings panel title', 'buddypress-auto-friendship-pro' ) );

		$section_misc = $misc_panel->add_section( 'settings', _x( 'Misc Settings', 'Admin settings section title', 'buddypress-auto-friendship-pro' ) );

		$fields = array(
			array(
				'name'    => 'invited_connection_types',
				'label'   => _x( 'On invited user joining site', 'Admin settings', 'buddypress-auto-friendship-pro' ),
				'desc'    => __( 'Connect with inviter user.', 'buddypress-auto-friendship-pro' ),
				'type'    => 'multicheck',
				'options' => bpafp_get_available_connection_type_options(),
				'default' => array(),
			),
		);

		$section_misc->add_fields( $fields );

		$desc = <<<EOT
You can use <a href='%s'>Auto Friendship Tools</a> page to auto connect any 2 member on your network.
EOT;
	$desc = sprintf( $desc, admin_url('tools.php?page=bp-auto-friendship'));

		$section_misc = $misc_panel->add_section( 'tools-link', _x( 'Tools Page', 'Admin settings section title', 'buddypress-auto-friendship-pro' ), $desc );

		$this->page = $page;

		// allow enabling options.
		$page->init();
	}

	/**
	 * Add Menu
	 */
	public function add_menu() {

		add_options_page(
			_x( 'Auto Friendship Pro', 'Admin settings page title', 'buddypress-auto-friendship-pro' ),
			_x( 'Auto Friendship Pro', 'Admin settings menu label', 'buddypress-auto-friendship-pro' ),
			'manage_options',
			$this->menu_slug,
			array( $this, 'render' )
		);
	}
}
