<?php
/**
 * Friendship list cloning helper
 *
 * @package    BuddyPress_Auto_Friendship_Pro
 * @subpackage Admin
 * @copyright  Copyright (c) 2020, Brajesh Singh
 * @license    https://www.gnu.org/licenses/gpl.html GNU Public License
 * @author     Brajesh Singh, Ravi Sharma
 * @since      1.0.0
 */

namespace BuddyPress_Auto_Friendship_Pro\Admin;

// Do not allow direct access over web.
defined( 'ABSPATH' ) || exit;

/**
 * Class Friendship_List_Clone_Admin_Helper
 */
class Friendship_List_Clone_Admin_Helper {

	/**
	 * Singleton.
	 *
	 * @var Friendship_List_Clone_Admin_Helper
	 */
	private static $instance = null;

	/**
	 * Post type attached to
	 *
	 * @var string
	 */
	private $post_type = '';

	/**
	 * Cloning Action
	 *
	 * @var string
	 */
	private $action = '';

	/**
	 * Friendship_List_Clone_Admin_Helper constructor.
	 *
	 * @param string $post_type Post type.
	 */
	private function __construct( $post_type ) {
		$this->post_type = $post_type;
		$this->action    = $post_type . '_clone';

		$this->setup();
	}

	/**
	 * Setup the bootstrapper
	 *
	 * @param string $post_type Post type attached to.
	 *
	 * @return Friendship_List_Clone_Admin_Helper
	 */
	public static function boot( $post_type ) {

		if ( is_null( self::$instance ) ) {
			self::$instance = new self( $post_type );
		}

		return self::$instance;
	}

	/**
	 * Setup callbacks
	 */
	private function setup() {
		add_filter( 'post_row_actions', array( $this, 'add_clone_link' ), 10, 2 );
		add_action( 'post_action_' . $this->action, array( $this, 'clone_post' ) );
	}

	/**
	 * Adds clone link as row action in post type list items
	 *
	 * @param string[] $actions Actions.
	 * @param \WP_Post $post    Post object.
	 *
	 * @return array
	 */
	public function add_clone_link( $actions, $post ) {

		if ( ! is_super_admin() ) {
			return $actions;
		}

		if ( $this->post_type !== $post->post_type ) {
			return $actions;
		}

		$post_type_object = get_post_type_object( $post->post_type );

		if ( ! $post_type_object ) {
			return $actions;
		}

		// Not sure.
		$clone_link = add_query_arg( 'action', $this->action, admin_url( sprintf( $post_type_object->_edit_link, $post->ID ) ) );

		$actions[ $this->action ] = sprintf(
			'<a href="%s" rel="bookmark">%s</a>',
			wp_nonce_url( $clone_link, "{$this->action}-post-{$post->ID}" ),
			__( 'Clone', 'buddypress-auto-friendship-pro' )
		);

		return $actions;
	}

	/**
	 * Clones form as a new post.
	 *
	 * @param int $post_id Post id.
	 */
	public function clone_post( $post_id ) {

		if ( ! is_super_admin() ) {
			return;
		}

		if ( ! isset( $_GET['_wpnonce'] ) || ! wp_verify_nonce( wp_unslash( $_GET['_wpnonce'] ), $this->action . '-post-' . $post_id ) ) {
			return;
		}

		$post_data = $this->get_clone_data( $post_id );

		if ( ! $post_data || $post_data['post_type'] !== $this->post_type ) {
			return;
		}

		if ( ! empty( $post_data['post_title'] ) ) {
			$post_data['post_title'] = $post_data['post_title'] . '-' . apply_filters( $this->post_type . '_cloned_post_title_suffix', __( 'Duplicate', 'buddypress-auto-friendship-pro' ) );
		}

		$meta_values = array();

		if ( isset( $post_data['meta'] ) ) {
			$meta_values = $post_data['meta'];

			unset( $post_data['meta'] );
		}

		$post_id = wp_insert_post( $post_data );

		if ( ! $post_id ) {
			return;
		}

		foreach ( $meta_values as $key => $value ) {
			update_post_meta( $post_id, $key, $value );
		}

		wp_safe_redirect( admin_url( 'post.php?post=' . $post_id . '&action=edit' ) );
		exit;
	}

	/**
	 * Get Clone data
	 *
	 * @param int $post_id Post id.
	 *
	 * @return array|null
	 */
	private function get_clone_data( $post_id ) {

		$post = get_post( $post_id );

		if ( ! $post ) {
			return null;
		}

		$details = array(
			'post_status'    => $post->post_status,
			'post_title'     => $post->post_title,
			'post_author'    => $post->post_author,
			'post_content'   => $post->post_content,
			'post_excerpt'   => $post->post_excerpt,
			'comment_status' => $post->comment_status,
			'post_password'  => $post->post_password,
			'post_parent'    => $post->post_parent,
			'post_type'      => $post->post_type,
		);

		$meta_values = get_post_custom( $post_id );

		$excluded_meta_keys = apply_filters( $this->post_type . '_clone_excluded_meta_keys', array( '_edit_lock', '_edit_last' ) );

		foreach ( $meta_values as $key => $value ) {

			if ( in_array( $key, $excluded_meta_keys, true ) ) {
				continue;
			}

			// Due to post custom need to pick first value of array.
			$details['meta'][ $key ] = maybe_unserialize( current( $value ) );
		}

		return $details;
	}
}
