<?php
/**
 * Bootstrapper. Initializes the plugin.
 *
 * @package    BuddyPress_Auto_Friendship_Pro
 * @subpackage Bootstrap
 * @copyright  Copyright (c) 2018, Brajesh Singh
 * @license    https://www.gnu.org/licenses/gpl.html GNU Public License
 * @author     Brajesh Singh
 * @since      1.0.0
 */

namespace BuddyPress_Auto_Friendship_Pro\Bootstrap;

use BuddyPress_Auto_Friendship_Pro\Admin\V1_Upgrader;
use BuddyPress_Auto_Friendship_Pro\Admin\BPAFP_Admin_Loader;
use BuddyPress_Auto_Friendship_Pro\Handlers\Actions_Handler;
use BuddyPress_Auto_Friendship_Pro\Handlers\Ajax_Handler;
use BuddyPress_Auto_Friendship_Pro\Core\Post_Type_Helper;
use BuddyPress_Auto_Friendship_Pro\Core\Automatic_Friendship_Creator;
use BuddyPress_Auto_Friendship_Pro\Core\Automatic_Friendship_Remover;
use BuddyPress_Auto_Friendship_Pro\Core\Follow_Following\Automatic_Follower_Creator;
use BuddyPress_Auto_Friendship_Pro\Core\Follow_Following\Automatic_Follower_Remover;
use BuddyPress_Auto_Friendship_Pro\Core\Follow_Following\Automatic_Following_Creator;
use BuddyPress_Auto_Friendship_Pro\Core\Follow_Following\Automatic_Following_Remover;
use BuddyPress_Auto_Friendship_Pro\Modules\Groups\Group_Module_Loader;

// Do not allow direct access over web.
defined( 'ABSPATH' ) || exit;

/**
 * Bootstrapper.
 */
class Bootstrapper {

	/**
	 * Setup the bootstrapper.
	 */
	public static function boot() {
		$self = new self();
		$self->setup();
	}

	/**
	 * Bind hooks
	 */
	private function setup() {
		add_action( 'bp_loaded', array( $this, 'load' ) );
		add_action( 'bp_init', array( $this, 'load_translations' ) );
		add_action( 'admin_init', array( $this, 'admin_init' ) );
		add_action( 'admin_notices', array( $this, 'render_notice' ) );
	}

	/**
	 * Load core functions/template tags.
	 * These are non auto loadable constructs.
	 */
	public function load() {

		$path = bp_auto_friendship_pro()->path;

		$files = array(
			'src/core/bpafp-functions.php',
		);

		foreach ( $files as $file ) {
			require_once $path . $file;
		}

		Actions_Handler::boot();
		Ajax_Handler::boot();
		Assets_Loader::boot();
		Post_Type_Helper::boot();
		Automatic_Friendship_Creator::boot();
		Automatic_Friendship_Remover::boot();

		if ( bpafp_is_active_follow_following() ) {
			Automatic_Follower_Creator::boot();
			Automatic_Follower_Remover::boot();
			Automatic_Following_Creator::boot();
			Automatic_Following_Remover::boot();
		}

		if ( is_admin() && ! wp_doing_ajax() ) {
			BPAFP_Admin_Loader::boot();
		}

		if ( bp_is_active( 'groups' ) ) {
			Group_Module_Loader::boot();
		}
	}

	/**
	 * Load translations.
	 */
	public function load_translations() {
		load_plugin_textdomain(
			'buddypress-auto-friendship-pro',
			false,
			basename( bp_auto_friendship_pro()->path ) . '/languages'
		);
	}

	/**
	 * Runs on admin initialization.
	 */
	public function admin_init() {

		if ( ! function_exists( 'buddypress' ) ) {
			return;
		}

		// does our db needs upgrade?
		if ( ! get_option( '_bpafp_db_verion' )  ) {
			V1_Upgrader::upgrade();
		}
	}

	public function render_notice() {

		if ( ! function_exists( 'bp_is_active' ) || bp_is_active( 'friends' ) || bpafp_is_active_follow_following() ) {
			return;
		}

		global $pagenow;
		$current_screen = get_current_screen();

		$render = false;
		if ( isset( $current_screen->post_type ) && bpafp_get_list_post_type() == $current_screen->post_type ) {
			$render = true;
		} elseif ( 'plugins.php' == $pagenow ) {
			$render = true;
		}

		if ( ! $render ) {
			return;
		}

		$message = __( 'Auto Friendship Pro: It seems both friendship and follow feature disabled!. Please check.', 'buddypress-auto-friendship-pro' );

		printf( '<div class="notice notice-warning is-dismissible"><p>%s</p></div>', esc_html( $message ) );
	}
}
