<?php
/**
 * Core functions file
 *
 * @package BuddyPress_Auto_Friendship_Pro
 */

use BuddyPress_Auto_Friendship_Pro\Core\User_List_Matcher;

// Do not allow direct access over web.
defined( 'ABSPATH' ) || exit;

/**
 * Get default settings
 *
 * @return array
 */
function bpafp_get_default_options() {
	$defaults = array(
		'auto_friendship_triggers' => array(
			'activate_account' => 'activate_account',
			'user_register'    => 'user_register',
		),
		'invited_connection_types' => array(
			'friend' => 'friend',
		),
	);

	return $defaults;
}

/**
 * Get setting option value
 *
 * @param string $key     Setting key.
 * @param mixed  $default Default value.
 *
 * @return mixed|null
 */
function bpafp_get_option( $key, $default = null ) {
	$settings = get_option( 'bp_auto_friendship_pro', bpafp_get_default_options() );

	if ( isset( $settings[ $key ] ) ) {
		return $settings[ $key ];
	}

	return $default;
}

/**
 * Get auto friendship triggers
 *
 * @return array
 */
function bpafp_get_auto_friendship_triggers() {
	$triggers = array(
		'activate_account'   => __( 'Account Activation', 'buddypress-auto-friendship-pro' ),
		'user_register'      => __( 'User Registration ( Supported in BuddyBoss & BuddyPress < 14.0 )', 'buddypress-auto-friendship-pro' ),
		'first_login'        => __( 'First Login', 'buddypress-auto-friendship-pro' ),
		'profile_update'     => __( 'On Profile Update', 'buddypress-auto-friendship-pro' ),
		'role_change'        => __( 'Role Update', 'buddypress-auto-friendship-pro' ),
		'member_type_change' => __( 'Member Type(Profile Type) Update', 'buddypress-auto-friendship-pro' ),
	);

	return $triggers;
}

/**
 * Check if Auto friendship trigger on account activation enabled?.
 *
 * @return bool
 */
function bpafp_is_enabled_auto_friendship_on_activation() {
	$triggers = (array) bpafp_get_option( 'auto_friendship_triggers', array( 'activate_account' ) );

	if ( in_array( 'activate_account', $triggers, true ) ) {
		return true;
	}

	return false;
}

/**
 * Check if Auto friendship trigger on user registration enabled?.
 *
 * @return bool
 */
function bpafp_is_enabled_auto_friendship_on_registration() {
	$triggers = (array) bpafp_get_option( 'auto_friendship_triggers', array( 'user_register' ) );

	$enabled = false;
	if ( in_array( 'user_register', $triggers, true ) ) {
		$enabled = true;
	}

	return (bool) apply_filters( 'bp_auto_friendship_pro_enable_on_user_register', $enabled );
}

/**
 * Check sif Auto friendship should trigger on user role update?.
 *
 * @return bool
 */
function bpafp_is_auto_friendship_enabled_on_profile_update() {
	$triggers = (array) bpafp_get_option( 'auto_friendship_triggers', array() );

	$enabled = false;
	if ( in_array( 'profile_update', $triggers, true ) ) {
		$enabled = true;
	}

	return (bool) apply_filters( 'bp_auto_friendship_pro_enable_on_profile_update', $enabled );
}

/**
 * Check sif Auto friendship should trigger on user role update?.
 *
 * @return bool
 */
function bpafp_is_auto_friendship_enabled_on_role_change() {
	$triggers = (array) bpafp_get_option( 'auto_friendship_triggers', array() );

	$enabled = false;
	if ( in_array( 'role_change', $triggers, true ) ) {
		$enabled = true;
	}

	return (bool) apply_filters( 'bp_auto_friendship_pro_enable_on_role_change', $enabled );
}

/**
 * Check sif Auto friendship should trigger on user role update?.
 *
 * @return bool
 */
function bpafp_is_auto_friendship_enabled_on_member_type_change() {
	$triggers = (array) bpafp_get_option( 'auto_friendship_triggers', array() );

	$enabled = false;
	if ( in_array( 'member_type_change', $triggers, true ) ) {
		$enabled = true;
	}

	return (bool) apply_filters( 'bp_auto_friendship_pro_enable_on_member_type_change', $enabled );
}

/**
 * Get list posttype
 *
 * @return string
 */
function bpafp_get_list_post_type() {
	return 'bpafp';
}

/**
 * Get list attached members
 *
 * @param int $list_id List id.
 *
 * @return array
 */
function bpafp_get_list_members( $list_id ) {
	$members = get_post_meta( $list_id, '_attached_members', true );

	return $members ? (array) $members : array();
}

/**
 * Update list attached members
 *
 * @param int   $list_id Friendship id.
 * @param array $members Members.
 *
 * @return int|bool
 */
function bpafp_update_list_members( $list_id, $members = array() ) {
	return update_post_meta( $list_id, '_attached_members', $members );
}

/**
 * Get attched conditions
 *
 * @param int $list_id Friendship id.
 *
 * @return array
 */
function bpafp_get_list_conditions( $list_id ) {
	$conditions = get_post_meta( $list_id, '_attached_conditions', true );

	return $conditions ? (array) $conditions : array();
}

/**
 * Update attached conditions
 *
 * @param int   $friendship_list_id Friendship id.
 * @param array $conditions Members.
 *
 * @return bool|int
 */
function bpafp_update_list_conditions( $friendship_list_id, $conditions = array() ) {
	return update_post_meta( $friendship_list_id, '_attached_conditions', $conditions );
}

/**
 * Get matched list
 *
 * @param string $context Context.
 *
 * @return WP_Post[]
 */
function bpafp_get_active_lists( $context = '' ) {

	if ( ! $context ) {
		$context = 'friends';
	}

	$args = array(
		'post_type'   => bpafp_get_list_post_type(),
		'post_status' => 'publish',
		'numberposts' => - 1,
	);

	$args['meta_key']     = '_bpafp_list_type';
	$args['meta_value']   = $context;
	$args['meta_compare'] = '=';

	return get_posts( $args );
}

/**
 * Get xprofile fields
 *
 * @return array
 */
function bpafp_get_xprofile_fields() {
	static $fields = array();

	if ( ! function_exists( 'bp_xprofile_get_groups' ) || ! empty( $fields ) ) {
		return $fields;
	}

	$groups = bp_xprofile_get_groups(
		array(
			'hide_empty_groups' => true,
			'fetch_fields'      => true,
		)
	);

	if ( empty( $groups ) ) {
		return array();
	}

	$fields = array();
	foreach ( $groups as $group ) {
		foreach ( $group->fields as $field ) {
			$fields[ $field->id ] = $field->name;
		}
	}

	return $fields;
}

/**
 * Get register member types
 *
 * @return array
 */
function bpafp_get_member_types() {
	$register_member_types = bp_get_member_types( array(), 'objects' );

	if ( empty( $register_member_types ) ) {
		return array();
	}

	$member_types = array();
	foreach ( $register_member_types as $register_member_type => $detail ) {
		$member_types[ $register_member_type ] = $detail->labels['name'];
	}

	return $member_types;
}

/**
 * Get raw field data.
 *
 * @param string $field field id or name.
 * @param int    $user_id user id.
 *
 * @return string|array
 */
function bpafp_get_raw_profile_data( $field, $user_id ) {

	if ( is_numeric( $field ) ) {
		$field_id = $field;
	} else {
		$field_id = xprofile_get_field_id_from_name( $field );
	}

	if ( empty( $field_id ) ) {
		return false;
	}

	return maybe_unserialize( BP_XProfile_ProfileData::get_value_byid( $field_id, $user_id ) );
}

/**
 * Check if follow following feature is enabled.
 *
 * @return bool
 */
function bpafp_is_active_follow_following() {
	$is_active = false;

	if ( function_exists( 'bp_follow_init' ) ) {
		$is_active = true;
	} elseif ( function_exists( 'bp_is_activity_follow_active' ) ) {
		$is_active = bp_is_activity_follow_active();
	}

	return (bool) $is_active;
}

/**
 * Get user matching friendship list ids.
 *
 * @param int       $user_id User ids.
 * @param WP_Post[] $lists   Friendship lists.
 *
 * @return array
 */
function bpafp_get_matching_friendship_list_ids( $user_id, $lists ) {

	if ( ! $user_id || ! $lists ) {
		return array();
	}

	$user_list_matcher = new User_List_Matcher( $user_id );

	return $user_list_matcher->get_matching_list_ids( $lists );
}

/**
 * Retrieves available connection types as options array
 *
 * @param string $context Options context(removal/add).
 *
 * @return array
 */
function bpafp_get_available_connection_type_options( $context = 'add' ) {
	$options = array();

	$removal = 'removal' === $context ? true : false;

	$options['friend'] = $removal ? __( 'Remove as Friend', 'buddypress-auto-friendship-pro' ) : __( 'Add as Friend', 'buddypress-auto-friendship-pro' );

	if ( bpafp_is_active_follow_following() ) {
		$options['follower']  = $removal ? __( 'Remove as Follower', 'buddypress-auto-friendship-pro' ) : __( 'Add as Follower', 'buddypress-auto-friendship-pro' );
		$options['following'] = $removal ? __( 'Remove as Following', 'buddypress-auto-friendship-pro' ) : __( 'Add as Following', 'buddypress-auto-friendship-pro' );
	}

	return $options;
}

/**
 * Retrieves user url
 *
 * @param int $user_id User id.
 *
 * @return string
 */
function bpafp_get_user_url( $user_id ) {

	if ( function_exists( 'bp_members_get_user_url' ) ) {
		return bp_members_get_user_url( $user_id );
	}

	return bp_core_get_user_domain( $user_id );
}
