<?php
/**
 * Class to create automatic friendship
 *
 * @package    BuddyPress_Auto_Friendship_Pro
 * @subpackage Core
 * @copyright  Copyright (c) 2020, Brajesh Singh
 * @license    https://www.gnu.org/licenses/gpl.html GNU Public License
 * @author     Ravi Sharma, Brajesh Singh
 * @since      1.0.0
 */

namespace BuddyPress_Auto_Friendship_Pro\Core;

// Exit if accessed directly.
defined( 'ABSPATH' ) || exit;

/**
 * Class Automatic_Friendship_Creator
 */
class Automatic_Friendship_Creator {

	/**
	 * Single instance.
	 *
	 * @var Automatic_Friendship_Creator
	 */
	private static $instance = null;

	/**
	 * Constructor.
	 */
	private function __construct() {
		$this->setup();
	}

	/**
	 * Factory method.
	 *
	 * @return Automatic_Friendship_Creator
	 */
	public static function get_instance() {

		if ( is_null( self::$instance ) ) {
			self::$instance = new self();
		}

		return self::$instance;
	}

	/**
	 * Boot class
	 */
	public static function boot() {
		return self::get_instance();
	}

	/**
	 * Setup actions
	 */
	private function setup() {
		add_action( 'bp_core_activated_user', array( $this, 'on_user_activate' ) );
		add_action( 'user_register', array( $this, 'on_registration' ) );
		add_action( 'xprofile_updated_profile', array( $this, 'on_profile_update' ) );
		add_action( 'set_user_role', array( $this, 'on_role_update' ) );
		add_action( 'add_user_role', array( $this, 'on_role_update' ) );

		add_action( 'bp_set_member_type', array( $this, 'on_member_type_update' ) );
		add_action( 'bp_remove_member_type', array( $this, 'on_member_type_update' ) );
	}

	/**
	 * On user activate
	 *
	 * @param int $user_id user id.
	 */
	public function on_user_activate( $user_id ) {

		if ( bpafp_is_enabled_auto_friendship_on_activation() ) {
			$this->add_friends( $user_id );
		}
	}

	/**
	 * Triggers matching on 'user_register' action.
	 *
	 * It only adds the global lists since xprofile data is not available.
	 *
	 * @param int $user_id user id.
	 */
	public function on_registration( $user_id ) {

		if ( bpafp_is_enabled_auto_friendship_on_registration() ) {
			$this->add_friends( $user_id );
		}
	}

	/**
	 * Runs on profile update.
	 *
	 * @param int $user_id User id.
	 */
	public function on_profile_update( $user_id ) {
		if ( bpafp_is_auto_friendship_enabled_on_profile_update() ) {
			$this->add_friends( $user_id );
		}
	}

	/**
	 * Runs auto friendship on role update.
	 *
	 * @param int $user_id user id.
	 */
	public function on_role_update( $user_id ) {
		if ( bpafp_is_auto_friendship_enabled_on_role_change() ) {
			$this->add_friends( $user_id );
		}
	}

	/**
	 * Runs auto friendship on member type update.
	 *
	 * @param int $user_id user id.
	 */
	public function on_member_type_update( $user_id ) {
		if ( bpafp_is_auto_friendship_enabled_on_member_type_change() ) {
			$this->add_friends( $user_id );
		}
	}

	/**
	 * Add friends based on the matching lists.
	 *
	 * @param int $user_id user id.
	 */
	public function add_friends( $user_id ) {
		// Return if friends module not enabled.
		if ( ! function_exists( 'friends_check_friendship' ) || ! $user_id ) {
			return;
		}

		$matching_list_ids = bpafp_get_matching_friendship_list_ids( $user_id, bpafp_get_active_lists( 'friends' ) );

		if ( ! $matching_list_ids ) {
			return;
		}

		foreach ( $matching_list_ids as $list_id ) {
			$this->create_user_friendships( $user_id, $list_id );
		}
	}

	/**
	 * Add connections form friendship lists
	 *
	 * @param int        $user_id User id.
	 * @param \WP_Post[] $lists Friendship List ids.
	 */
	public static function create_connections( $user_id, $lists ) {

		if ( ! function_exists( 'friends_check_friendship' ) || ! $user_id ) {
			return;
		}

		$matching_list_ids = bpafp_get_matching_friendship_list_ids( $user_id, $lists );

		if ( ! $matching_list_ids ) {
			return;
		}

		$self = self::get_instance();

		foreach ( $matching_list_ids as $list_id ) {
			$self->create_user_friendships( $user_id, $list_id );
		}
	}

	/**
	 * Add list members as friends
	 *
	 * @param int $user_id User id.
	 * @param int $list_id List id.
	 */
	public static function add_list_members_as_friends( $user_id, $list_id ) {
		$self = self::boot();

		$self->create_user_friendships( $user_id, $list_id );
	}

	/**
	 * Sync user friend list
	 *
	 * @param int  $user_id   User id.
	 * @param int  $list_id   List id.
	 * @param bool $hard_sync Hard sync enabled or not.
	 */
	public static function sync( $user_id, $list_id, $hard_sync = false ) {

		if ( ! $user_id || ! $list_id || ! function_exists( 'friends_get_friend_user_ids' ) ) {
			return;
		}

		$self = self::boot();

		$list_members = bpafp_get_list_members( $list_id );

		if ( $hard_sync ) {
			// remove the friends who are not in the list.
			$friend_ids           = friends_get_friend_user_ids( $user_id );
			$removable_friend_ids = $friend_ids ? array_diff( $friend_ids, $list_members ) : array();

			foreach ( $removable_friend_ids as $friend_id ) {
				// As friend initiate the request or user has initiated the request.
				friends_remove_friend( $friend_id, $user_id );
			}
		}

		$self->create_user_friendships( $user_id, $list_id );
	}

	/**
	 * Create friendship
	 *
	 * @param int $user_id User id.
	 * @param int $friendship_list_id Friendship id.
	 */
	private function create_user_friendships( $user_id, $friendship_list_id ) {
		$user_id = (int) $user_id;

		if ( ! function_exists( 'friends_check_friendship' ) ) {
			return;
		}

		$attached_members = bpafp_get_list_members( $friendship_list_id );

		if ( ! $attached_members ) {
			return;
		}

		foreach ( $attached_members as $member ) {
			$user = get_user_by( 'id', $member );

			if ( ! $user || friends_check_friendship( $user_id, $member ) ) {
				continue;
			}

			friends_add_friend( $member, $user_id, true );
		}
	}
}
