<?php
/**
 * Class to remove friendship automatic
 *
 * @package    BuddyPress_Auto_Friendship_Pro
 * @subpackage Core
 * @copyright  Copyright (c) 2020, Brajesh Singh
 * @license    https://www.gnu.org/licenses/gpl.html GNU Public License
 * @author     Ravi Sharma, Brajesh Singh
 * @since      1.0.0
 */

namespace BuddyPress_Auto_Friendship_Pro\Core;

// Exit if accessed directly.
defined( 'ABSPATH' ) || exit;

/**
 * Class Automatic_Friendship_Remover
 */
class Automatic_Friendship_Remover {

	/**
	 * Single instance.
	 *
	 * @var Automatic_Friendship_Remover
	 */
	private static $instance = null;

	/**
	 * Constructor.
	 */
	private function __construct() {
		$this->setup();
	}

	/**
	 * Factory method.
	 *
	 * @return Automatic_Friendship_Remover
	 */
	public static function get_instance() {

		if ( is_null( self::$instance ) ) {
			self::$instance = new self();
		}

		return self::$instance;
	}

	/**
	 * Boot class
	 */
	public static function boot() {
		return self::get_instance();
	}

	/**
	 * Setup actions
	 */
	private function setup() {
		// As no friend user have on activation or registration.
		// add_action( 'bp_core_activated_user', array( $this, 'on_user_activate' ) );
		// add_action( 'user_register', array( $this, 'on_registration' ) );

		add_action( 'xprofile_updated_profile', array( $this, 'on_profile_update' ) );
		add_action( 'set_user_role', array( $this, 'on_role_update' ) );
		add_action( 'add_user_role', array( $this, 'on_role_update' ) );

		add_action( 'bp_set_member_type', array( $this, 'on_member_type_update' ) );
		add_action( 'bp_remove_member_type', array( $this, 'on_member_type_update' ) );
	}

	/**
	 * On user activate
	 *
	 * @param int $user_id user id.
	 */
	public function on_user_activate( $user_id ) {}

	/**
	 * Triggers matching on 'user_register' action.
	 *
	 * It only adds the global lists since xprofile data is not available.
	 *
	 * @param int $user_id user id.
	 */
	public function on_registration( $user_id ) {}

	/**
	 * Runs on profile update.
	 *
	 * @param int $user_id User id.
	 */
	public function on_profile_update( $user_id ) {
		if ( bpafp_is_auto_friendship_enabled_on_profile_update() ) {
			$this->remove_friends( $user_id );
		}
	}

	/**
	 * Runs auto friendship on role update.
	 *
	 * @param int $user_id user id.
	 */
	public function on_role_update( $user_id ) {
		if ( bpafp_is_auto_friendship_enabled_on_role_change() ) {
			$this->remove_friends( $user_id );
		}
	}

	/**
	 * Runs auto friendship on member type update.
	 *
	 * @param int $user_id user id.
	 */
	public function on_member_type_update( $user_id ) {
		if ( bpafp_is_auto_friendship_enabled_on_member_type_change() ) {
			$this->remove_friends( $user_id );
		}
	}

	/**
	 * Add friends based on the matching lists.
	 *
	 * @param int $user_id user id.
	 */
	public function remove_friends( $user_id ) {
		// Return if friends module not enabled.
		if ( ! function_exists( 'friends_check_friendship' ) || ! $user_id ) {
			return;
		}

		$matching_list_ids = bpafp_get_matching_friendship_list_ids( $user_id, bpafp_get_active_lists( 'unfriend' ) );

		if ( ! $matching_list_ids ) {
			return;
		}

		foreach ( $matching_list_ids as $list_id ) {
			$this->delete_user_friendships( $user_id, $list_id );
		}
	}

	/**
	 * Remove connections form friendship lists
	 *
	 * @param int        $user_id User id.
	 * @param \WP_Post[] $lists Friendship List ids.
	 */
	public static function remove_connections( $user_id, $lists ) {

		if ( ! function_exists( 'friends_check_friendship' ) || ! $user_id ) {
			return;
		}

		$matching_list_ids = bpafp_get_matching_friendship_list_ids( $user_id, $lists );

		if ( ! $matching_list_ids ) {
			return;
		}

		$self = self::get_instance();

		foreach ( $matching_list_ids as $list_id ) {
			$self->delete_user_friendships( $user_id, $list_id );
		}
	}

	/**
	 * Remove list members as friends
	 *
	 * @param int $user_id User id.
	 * @param int $list_id List id.
	 */
	public static function remove_list_members_as_friends( $user_id, $list_id ) {
		$self = self::boot();

		$self->delete_user_friendships( $user_id, $list_id );
	}

	/**
	 * Sync user unfriend list
	 *
	 * @param int  $user_id   User id.
	 * @param int  $list_id   List id.
	 * @param bool $harh_sync Hard sync enabled or not.
	 */
	public static function sync( $user_id, $list_id, $harh_sync = false ) {

		if ( ! $user_id || ! $list_id ) {
			return;
		}

		$self = self::boot();

		$self->delete_user_friendships( $user_id, $list_id );
	}

	/**
	 * Delete friendship
	 *
	 * @param int $user_id User id.
	 * @param int $friendship_list_id Friendship id.
	 */
	private function delete_user_friendships( $user_id, $friendship_list_id ) {

		if ( ! function_exists( 'friends_check_friendship' ) ) {
			return;
		}

		$attached_member_ids = bpafp_get_list_members( $friendship_list_id );

		if ( ! $attached_member_ids ) {
			return;
		}

		foreach ( $attached_member_ids as $member_id ) {
			$user = get_user_by( 'id', $member_id );

			if ( $user && friends_check_friendship( $user_id, $member_id ) ) {
				friends_remove_friend( $member_id, $user_id );
			}
		}
	}
}
