<?php
/**
 * Matcher class
 *
 * @package    BuddyPress_Auto_Friendship_Pro
 * @subpackage Core
 * @copyright  Copyright (c) 2020, Brajesh Singh
 * @license    https://www.gnu.org/licenses/gpl.html GNU Public License
 * @author     Ravi Sharma, Brajesh Singh
 * @since      1.0.0
 */

namespace BuddyPress_Auto_Friendship_Pro\Core;

// Exit if accessed directly.
defined( 'ABSPATH' ) || exit;

/**
 * Class Matcher
 */
class Matcher {

	/**
	 * User id
	 *
	 * @var int
	 */
	private $user_id = 0;

	/**
	 * Matcher constructor.
	 *
	 * @param int $user_id User id.
	 */
	protected function __construct( $user_id ) {
		$this->user_id = $user_id;
	}

	/**
	 * Check if list is matched or not
	 *
	 * @param \WP_Post $list List object.
	 *
	 * @return bool
	 */
	protected function is_list_match( $list ) {
		// If empty post id or not our post type return false.
		if ( empty( $list->ID ) || empty( $list->post_type ) || $list->post_type !== bpafp_get_list_post_type() ) {
			return false;
		}

		$conditions = bpafp_get_list_conditions( $list->ID );

		// A list without any condition is a match.
		if ( ! $conditions ) {
			return true;
		}

		// By default, we assume the list to be a match.
		$is_match = true;

		foreach ( $conditions as $condition ) {

			if ( ! isset( $condition['id'], $condition['op'], $condition['value'] ) ) {
				$is_match = false;
				break;
			}

			if ( strpos( $condition['id'], 'xprofile_' ) !== false ) {
				$is_match = $this->match_xprofile_condition( $condition );
			} elseif ( 'role' === $condition['id'] ) {
				$is_match = $this->match_role_condition( $condition );
			} elseif ( 'member_type' === $condition['id'] ) {
				$is_match = $this->match_member_type_condition( $condition );
			} else {
				$is_match = false;
			}

			// if it is not a match, break.
			if ( ! $is_match ) {
				break;
			}
		}

		return $is_match;
	}

	/**
	 * Match value
	 *
	 * @param mixed  $data Match value.
	 * @param mixed  $values Matching value.
	 * @param string $operator Operator.
	 *
	 * @return bool
	 */
	private function match_value( $data, $values, $operator ) {
		$is_match = false;

		// Note: we are using loose comparision oeprator intentionally, please do not change.
		switch ( $operator ) {
			case '=':
				$is_match = $data == $values;
				break;

			case '!=':
				$is_match = $data != $values;
				break;

			case 'NOT IN':
				$values = array_map( 'trim', explode( ',', $values ) );

				if ( is_array( $data ) ) {
					$is_match = array_intersect( $data, $values ) ? false : true;
				} else {
					$is_match = ! in_array( $data, $values );
				}

				break;

			case 'IN':
				$values = array_map( 'trim', explode( ',', $values ) );

				if ( is_array( $data ) ) {
					$is_match = array_intersect( $data, $values ) ? true : false;
				} else {
					$is_match = in_array( $data, $values );
				}

				break;

			default:
				$is_match = false;
				break;
		}

		return $is_match;
	}

	/**
	 * Prepares field data
	 *
	 * @param int          $field_id Field id.
	 * @param string|array $field_data Field data.
	 *
	 * @return string|array
	 */
	private function prepare_data( $field_id, $field_data ) {

		if ( ! defined( 'BP_PLATFORM_VERSION' ) ) {
			return $field_data;
		}

		$field = xprofile_get_field( $field_id );

		if ( $field && 'membertypes' === $field->type && is_numeric( $field_data ) ) {
			// Buddyboss stores post_id as field value for membertypes field type.
			$field_data = get_post_meta( absint( $field_data ), '_bp_member_type_key', true );
		}

		return $field_data;
	}

	/**
	 * Prepares values for gender field by substituting the provided value on BuddyBoss platform.
	 *
	 * @param int    $field_id Field id.
	 * @param string $condition_value Field value.
	 *
	 * @return string
	 */
	private function prepare_value( $field_id, $condition_value ) {

		if ( ! $condition_value || ! in_array( strtolower( $condition_value ), array( 'male', 'female' ), true ) ) {
			return $condition_value;
		}

		if ( ! defined( 'BP_PLATFORM_VERSION' ) ) {
			return $condition_value;
		}

		$field = xprofile_get_field( $field_id );

		if ( 'gender' === $field->type ) {
			$condition_value = $this->prepare_gender_value( $condition_value );
		}

		return $condition_value;
	}

	/**
	 * Substitute the values of valid for gender on BuddyBoss platform.
	 *
	 * @param string $value string.
	 *
	 * @return string
	 */
	private function prepare_gender_value( $value ) {

		$match_value = strtolower( $value );

		if ( 'male' === $match_value ) {
			$value = 'his_Male';
		} elseif ( 'female' === $match_value ) {
			$value = 'her_Female';
		}

		return $value;
	}

	/**
	 * Match xprofile condition
	 *
	 * @param array $condition Condition array.
	 *
	 * @return bool
	 */
	private function match_xprofile_condition( $condition ) {

		if ( empty( $condition['value'] ) ) {
			return false;
		}

		$field_id = str_replace( 'xprofile_', '', $condition['id'] );

		$field_data = bpafp_get_raw_profile_data( $field_id, $this->user_id );
		$field_data = $this->prepare_data( $field_id, $field_data );

		if ( '0' !== $field_data && empty( $field_data ) ) {
			return false;
		}

		$condition_value = $this->prepare_value( $field_id, $condition['value'] );

		return $this->match_value( $field_data, $condition_value, $condition['op'] );
	}
	/**
	 * Match role condition
	 *
	 * @param array $condition Condition array.
	 *
	 * @return bool
	 */
	private function match_role_condition( $condition ) {

		if ( empty( $condition['value'] ) ) {
			return false;
		}

		$user = get_user_by( 'id', $this->user_id );

		if ( ! $user ) {
			return false;
		}

		$roles = $user->roles;

		if ( $roles && $this->needs_scalar_operand( $condition['op'] ) ) {
			$roles = array_pop( $roles );
		}

		return $this->match_value( $roles, $condition['value'], $condition['op'] );
	}

	/**
	 * Match member type condition
	 *
	 * @param array $condition Condition array.
	 *
	 * @return bool
	 */
	private function match_member_type_condition( $condition ) {

		if ( empty( $condition['value'] ) ) {
			return false;
		}

		$single = $this->needs_scalar_operand( $condition['op'] );

		$member_types = bp_get_member_type( $this->user_id, $single );

		if ( empty( $member_types ) ) {
			$member_types = 'null';
		}

		return $this->match_value( $member_types, $condition['value'], $condition['op'] );
	}

	/**
	 * Does this operator needs single valued.
	 *
	 * @param string $op operator.
	 *
	 * @return bool
	 */
	private function needs_scalar_operand( $op ) {
		return '=' === $op || '!=' === $op;
	}
}

