<?php
/**
 * Post type helper class
 *
 * @package    BuddyPress_Auto_Friendship_Pro
 * @subpackage Core
 * @copyright  Copyright (c) 2020, Brajesh Singh
 * @license    https://www.gnu.org/licenses/gpl.html GNU Public License
 * @author     Brajesh Singh, Ravi Sharma(raviousprime)
 * @since      1.0.0
 */

namespace BuddyPress_Auto_Friendship_Pro\Core;

// Exit if accessed directly.
defined( 'ABSPATH' ) || exit;

/**
 * Class Post_Type_Helper
 */
class Post_Type_Helper {

	/**
	 * Post type info
	 *
	 * @var string
	 */
	private $post_type = '';

	/**
	 * Boot class
	 */
	public static function boot() {
		$self = new self();

		$self->setup();
	}

	/**
	 * Setup actions
	 */
	private function setup() {
		$this->post_type = bpafp_get_list_post_type();

		add_action( 'init', array( $this, 'register_post_types' ) );
		add_action( "add_meta_boxes_{$this->post_type}", array( $this, 'add_meta_boxes' ) );
		add_action( "save_post_{$this->post_type}", array( $this, 'on_save' ) );
	}

	/**
	 * Register post type
	 */
	public function register_post_types() {
		$args = array(
			'labels'             => array(
				'name'               => _x( 'Friendship Lists', 'Post type general name', 'buddypress-auto-friendship-pro' ),
				'singular_name'      => _x( 'Friendship List', 'Post type singular name', 'buddypress-auto-friendship-pro' ),
				'add_new'            => __( 'Add New List', 'buddypress-auto-friendship-pro' ),
				'add_new_item'       => __( 'Add New List', 'buddypress-auto-friendship-pro' ),
				'edit_item'          => __( 'Edit List', 'buddypress-auto-friendship-pro' ),
				'view_item'          => __( 'View List', 'buddypress-auto-friendship-pro' ),
				'view_items'         => __( 'View Lists', 'buddypress-auto-friendship-pro' ),
				'search_items'       => __( 'Search Lists', 'buddypress-auto-friendship-pro' ),
				'not_found'          => __( 'No Lists found', 'buddypress-auto-friendship-pro' ),
				'not_found_in_trash' => __( 'No Lists found', 'buddypress-auto-friendship-pro' ),

			),
			'public'             => false,
			'publicly_queryable' => false,
			'show_in_menu'       => 'users.php',
			'capability_type'    => 'post',
			'supports'           => array( 'title' ),
			'show_ui'            => is_super_admin(),

		);

		register_post_type( $this->post_type, $args );
	}

	/**
	 * Add meta box
	 *
	 * @param \WP_Post $post Post object.
	 */
	public function add_meta_boxes( $post ) {
		add_meta_box(
			'bpafp_attached_members',
			__( 'List Members', 'buddypress-auto-friendship-pro' ),
			array( $this, 'render_attached_members_screen' ),
			$this->post_type,
			'normal',
			'default',
			$post
		);

		add_meta_box(
			'bpafp_attached_conditions',
			__( 'Conditions', 'buddypress-auto-friendship-pro' ),
			array( $this, 'render_attached_conditions_screen' ),
			$this->post_type,
			'normal',
			'default',
			$post
		);

		add_meta_box(
			'bpafp_list_type',
			__( 'List Type', 'buddypress-auto-friendship-pro' ),
			array( $this, 'render_list_type_meta_box' ),
			$this->post_type,
			'side',
			'high',
			$post
		);

		add_meta_box(
			'bpafp-list-sync-existing-users-metabox',
			__( 'Synchronise Existing Users', 'buddypress-auto-friendship-pro' ),
			array( $this, 'render_list_sync_users_button' ),
			$this->post_type,
			'normal',
			'default',
			$post
		);
	}

	/**
	 * Get conditions
	 *
	 * @return array
	 */
	private function get_attached_conditions() {

		if ( ! isset( $_POST['bpafp-available-conditions'], $_POST['bpafp-condition-operators'], $_POST['bpafp-condition-values'] ) ) {
			return array();
		}

		$available_conditions = wp_unslash( $_POST['bpafp-available-conditions'] );
		$condition_operators  = wp_unslash( $_POST['bpafp-condition-operators'] );
		$condition_values     = wp_unslash( $_POST['bpafp-condition-values'] );

		$available_conditions_count = count( $available_conditions );

		if ( $available_conditions_count !== count( $condition_operators ) ) {
			return array();
		} elseif ( $available_conditions_count !== count( $condition_values ) ) {
			return array();
		}

		$conditions = array();
		for ( $i = 0; $i < $available_conditions_count; $i ++ ) {

			$existed_key = array_search( $available_conditions[ $i ], array_column( $conditions, 'id' ) );

			// Empty value or id existed and value of operator is same.
			if ( empty( $condition_values[ $i ] ) || ( $existed_key !== false && $conditions[ $existed_key ]['op'] == $condition_operators[ $i ] ) ) {
				continue;
			}

			$conditions[] = array(
				'id'    => $available_conditions[ $i ],
				'op'    => $condition_operators[ $i ],
				'value' => $condition_values[ $i ],
			);
		}

		return $conditions;
	}

	/**
	 * On save post
	 *
	 * @param int $post_id Post id.
	 */
	public function on_save( $post_id ) {

		if ( empty( $_POST ) ) {
			return;
		}

		$attached_members    = isset( $_POST['bpafp-attached-members'] ) ? array_unique( $_POST['bpafp-attached-members'] ) : array();
		$attached_conditions = $this->get_attached_conditions();

		bpafp_update_list_members( $post_id, $attached_members );
		bpafp_update_list_conditions( $post_id, $attached_conditions );

		$list_type = isset( $_POST['bpafp-list-type'] ) ? sanitize_text_field( wp_unslash( $_POST['bpafp-list-type'] ) ) : '';
		update_post_meta( $post_id, '_bpafp_list_type', $list_type );
	}

	/**
	 * Render attached members screen
	 *
	 * @param \WP_Post $post Post object.
	 */
	public function render_attached_members_screen( $post ) {
		$attached_members = bpafp_get_list_members( $post->ID );

		?>
		<p>
			<input type="text" name="bpafp-members-autocomplete" id="bpafp-members-autocomplete" placeholder="<?php _e( 'Add members here.', 'buddypress-auto-friendship-pro' ); ?>" />
		</p>

			<ul id="bpafp-attached-members" class="bpafp-attached-members item-list">
				<?php if ( $attached_members ) : ?>
					<?php foreach ( $attached_members as $member_id ) : ?>
						<li class="vcard">
							<input type="hidden" value="<?php echo esc_attr( $member_id ); ?>" name="bpafp-attached-members[]" />
							<div class="item-avatar">
								<a href="<?php echo esc_url( bpafp_get_user_url( $member_id ) ); ?>">
									<?php
                                    echo bp_core_fetch_avatar( array(
									        'item_id' => $member_id,
                                            'width' => 60,
                                            'height' => 60,
                                        ) );
                                    ?>
								</a>
							</div>

							<div class="item">
								<div class="item-title fn">
									<a target="_blank" href="<?php echo esc_url( bpafp_get_user_url( $member_id ) ); ?>">
										<?php echo bp_core_get_user_displayname( $member_id ); ?>
									</a>
								</div>
							</div>
							<span title="Remove" class="bpafp-remove-member">x</span>
						</li>
					<?php endforeach; ?>
				<?php endif; ?>
			</ul>

        <p class="bpafp-list-type-desc"><?php _e( 'Search and Add members to the list here. These members will be added as Friends, Followers or Following based on list type.', 'buddypress-auto-friendship-pro' ); ?></p>
		<?php
	}

	/**
	 * Render attached conditions screen
	 *
	 * @param \WP_Post $post Post object.
	 */
	public function render_attached_conditions_screen( $post ) {
		$attached_conditions = bpafp_get_list_conditions( $post->ID );
		$fields              = bpafp_get_xprofile_fields();

		?>
            <div class="bpafp-attached-conditions-wrap">
                <?php if ( $attached_conditions ) : ?>
					<?php foreach ( $attached_conditions as $condition ) : ?>
	                    <div class="bpafp-attached-condition">
			                <select name="bpafp-available-conditions[]">
				                <option value=""><?php _e( '-- Select condition --', 'buddypress-auto-friendship-pro' ); ?></option>
				                <?php if( $fields ) : ?>
                                <optgroup label="<?php _e( 'Xprofile Fields', 'buddypress-auto-friendship-pro' ); ?>">
					                <?php foreach ( $fields as $field_id => $name ) : ?>
						                <option value="xprofile_<?php echo esc_attr( $field_id ); ?>" <?php selected( $condition['id'], "xprofile_{$field_id}" )?>><?php echo esc_html( $name ); ?></option>
					                <?php endforeach; ?>
				                </optgroup>
					            <?php endif; ?>
                                <optgroup label="<?php _e( 'Roles', 'buddypress-auto-friendship-pro' ); ?>">
                                    <option value="role" <?php selected( $condition['id'], "role" )?>><?php _e( 'Role', 'buddypress-auto-friendship-pro' ); ?></option>
                                </optgroup>
				                <optgroup label="<?php _e( 'Member Types(Profile Types)', 'buddypress-auto-friendship-pro' ); ?>">
					                <option value="member_type" <?php selected( $condition['id'], "member_type" )?>><?php _e( 'Member Type', 'buddypress-auto-friendship-pro' ); ?></option>
				                </optgroup>
			                </select>
			                <select name="bpafp-condition-operators[]">
                                <option value=""><?php _e( '-- Select operator --', 'buddypress-auto-friendship-pro' ); ?></option>
                                <?php foreach ( $this->get_conditions_operators() as $op => $label ) : ?>
					                <option value="<?php echo esc_attr( $op ); ?>" <?php selected( $condition['op'], $op )?>><?php echo esc_html( $label ); ?></option>
				                <?php endforeach; ?>
			                </select>
			                <input type="text" class="bpafp-condition-values" name="bpafp-condition-values[]" value="<?php echo esc_attr( $condition['value'] );?>"/>
	                        <span class="bpafp-remove-condition">x</span>
	                    </div>
					<?php endforeach; ?>
                <?php else: ?>
		            <div class="bpafp-attached-condition">
		                <select name="bpafp-available-conditions[]">
			                <option value=""><?php _e( '-- Select condition --', 'buddypress-auto-friendship-pro' ); ?></option>
			                <?php if( $fields ) : ?>
                            <optgroup label="<?php _e( 'Xprofile Fields', 'buddypress-auto-friendship-pro' ); ?>">
		                    <?php foreach ( $fields as $field_id => $name ) : ?>
		                        <option value="xprofile_<?php echo esc_attr( $field_id ); ?>"><?php echo esc_html( $name ); ?></option>
		                    <?php endforeach; ?>
		                    </optgroup>
			                <?php endif; ?>
                            <optgroup label="<?php _e( 'Roles', 'buddypress-auto-friendship-pro' ); ?>">
                                <option value="role"><?php _e( 'Role', 'buddypress-auto-friendship-pro' ); ?></option>
                            </optgroup>
			                <optgroup label="<?php _e( 'Member Types(Profile Types)', 'buddypress-auto-friendship-pro' ); ?>">
				                <option value="member_type"><?php _e( 'Member Type', 'buddypress-auto-friendship-pro' ); ?></option>
			                </optgroup>
		                </select>
		                <select name="bpafp-condition-operators[]">
                            <option value=""><?php _e( '-- Select operator --', 'buddypress-auto-friendship-pro' ); ?></option>
			                <?php foreach ( $this->get_conditions_operators() as $op => $label ) : ?>
				                <option value="<?php echo esc_attr( $op ); ?>"><?php echo esc_html( $label ); ?></option>
			                <?php endforeach; ?>
		                </select>
		                <input type="text" class="bpafp-condition-values" name="bpafp-condition-values[]" placeholder="<?php _e( 'comman separated values in case of multiple.', 'buddypress-auto-friendship-pro' ); ?>" />
                        <span class="bpafp-remove-condition">x</span>
		            </div>
                <?php endif; ?>
            </div>
		<button id="bpafp-add-new-condition" class="button"><?php _e( 'Add New', 'buddypress-auto-friendship-pro' ); ?></button>
        <?php
    }

	/**
     * Render list type meta box
     *
	 * @param \WP_Post $post Post object.
	 */
	public function render_list_type_meta_box( $post ) {
		$list_type = get_post_meta( $post->ID, '_bpafp_list_type', true );
		$list_type = $list_type ? $list_type : 'friends';

		?>
		<?php if ( bp_is_active( 'friends' ) || bpafp_is_active_follow_following() ) : ?>
            <div class="bpafp-list-type-wrap">
				<?php foreach ( $this->get_list_types() as $type => $detail ) : ?>
                    <label>
                        <input type="radio" name="bpafp-list-type"
                               value="<?php echo esc_attr( $type ); ?>" <?php checked( $list_type, $type ); ?>/>
						<?php echo $detail['label']; ?>
                    </label>
                    <p class="bpafp-list-type-desc"><?php echo $detail['desc']; ?></p>
				<?php endforeach; ?>
            </div>
		<?php else: ?>
            <p class="bpafp-list-type-desc"><?php esc_html_e( 'Please enable friends or follow feature first.', 'buddypress-auto-friendship-pro' ); ?></p>
		<?php endif; ?>
		<?php
	}

	/**
	 * Render sync existing users button
	 *
	 * @param \WP_Post $post Post object.
	 */
	public function render_list_sync_users_button( $post ) {
		$disabled = $post->post_status == 'publish' ? false : true;

		$message = $disabled ? __( 'To enable this action, Please publish the list.', 'buddypress-auto-friendship-pro' ) : __( 'Users matching the conditions from this list(saved conditions) will be added as friends, followers or following based on list type.', 'buddypress-auto-friendship-pro' );

		global $wpdb;
		$total_users = $wpdb->get_var( "SELECT COUNT(ID) FROM {$wpdb->users}" );

		?>

        <div class="bpafp-sync-friends-section">

            <ul id="bpafp-sync-status" class="bpafp-clearfix">
                <li><?php _e( 'Total users: ', 'buddypress-auto-friendship-pro' ) ?><span id="bpafp-total"><?php echo  $total_users ?></span></li>
                <li><?php _e( 'Processed: ', 'buddypress-auto-friendship-pro' ) ?><span id="bpafp-processed"></span></li>
                <li><?php _e( 'Skipped: ', 'buddypress-auto-friendship-pro' ) ?><span id="bpafp-skipped"></span></li>
                <li><?php _e( 'Remaining: ', 'buddypress-auto-friendship-pro' ) ?><span id="bpafp-remaining"></span></li>
            </ul>

            <p>
                <label for="bpafp-enable-hard-sync"><?php _e( 'Enable hard user sync: ', 'buddypress-auto-friendship-pro' ) ?></label>
                <input type="checkbox" value="1" id="bpafp-enable-hard-sync" <?php disabled( $disabled, true ); ?>></p>
            <p class="bpafp-list-type-desc"><?php _e( 'If, enabled Those users not matched with the list criteria will be unfriended or unfollow with list members.', 'buddypress-auto-friendship-pro' ); ?></p>
            <p>
                <button id="bpafp-sync-friends" class="button bpafp-sync-friends" data-list-id="<?php echo $post->ID; ?>" data-hard-sync="0" data-nonce="<?php echo wp_create_nonce( 'bpafp-sync-friends' ); ?>" <?php disabled( $disabled, true ); ?>><?php esc_html_e( 'Sync Users', 'buddypress-auto-friendship-pro' ); ?><span class="loading"><img src="<?php echo admin_url( 'images/loading.gif' ); ?>" /></span></button>
            </p>
            <p class="bpafp-list-type-desc"><?php echo $message; ?></p>
        </div>
		<?php
	}

	/**
	 * Get list types
	 *
	 * @return array
	 */
	private function get_list_types() {

		$list_types = array();

		if ( bp_is_active( 'friends' ) ) {
			$list_types['friends'] = array(
				'label' => __( 'Add as Friends', 'buddypress-auto-friendship-pro' ),
				'desc'  => __( 'List members will be added as friends of the registered user.', 'buddypress-auto-friendship-pro' ),
			);

			$list_types['unfriend'] = array(
				'label' => __( 'Remove as Friends', 'buddypress-auto-friendship-pro' ),
				'desc'  => __( 'List members will be removed as friends of the registered user.', 'buddypress-auto-friendship-pro' ),
			);
		}

		if ( bpafp_is_active_follow_following() ) {
            // 'followers', 'followings' was a bad choice for name initially bt we are stuck with it now.
            // the type should have been verb here to be more readable.
			$list_types['followers'] = array(
				'label' => __( 'Add as Followers', 'buddypress-auto-friendship-pro' ),
				'desc'  => __( 'List members will be added as followers of the registered user.', 'buddypress-auto-friendship-pro' ),
			);

			$list_types['unfollow'] = array(
				'label' => __( 'Remove as Followers', 'buddypress-auto-friendship-pro' ),
				'desc'  => __( 'List members will be made to unfollow the registered user.', 'buddypress-auto-friendship-pro' ),
			);

			$list_types['followings'] = array(
				'label' => __( 'Add as Following', 'buddypress-auto-friendship-pro' ),
				'desc'  => __( 'User will follow the list members.', 'buddypress-auto-friendship-pro' ),
			);

			$list_types['unfollowing'] = array(
				'label' => __( 'Remove as Following', 'buddypress-auto-friendship-pro' ),
				'desc'  => __( 'User will unfollow the list members.', 'buddypress-auto-friendship-pro' ),
			);
		}

		return $list_types;
	}

	/**
	 * Get available operator
	 *
	 * @return array
	 */
	private function get_conditions_operators() {
		return array(
			'='      => __( 'Equal to', 'buddypress-auto-friendship-pro' ),
			'!='     => __( 'Not equal to', 'buddypress-auto-friendship-pro' ),
			'IN'     => __( 'In', 'buddypress-auto-friendship-pro' ),
			'NOT IN' => __( 'Not in', 'buddypress-auto-friendship-pro' ),
		);
	}
}
