<?php
/**
 * Class to create automatic followers
 *
 * @package    BuddyPress_Auto_Friendship_Pro
 * @subpackage Core\Follow_Following
 * @copyright  Copyright (c) 2020, Brajesh Singh
 * @license    https://www.gnu.org/licenses/gpl.html GNU Public License
 * @author     Ravi Sharma, Brajesh Singh
 * @since      1.0.0
 */

namespace BuddyPress_Auto_Friendship_Pro\Core\Follow_Following;

use BuddyPress_Auto_Friendship_Pro\Core\User_List_Matcher;

// Exit if accessed directly.
defined( 'ABSPATH' ) || exit;

/**
 * Class Automatic_Follower_Creator
 */
class Automatic_Follower_Creator {

	use Follow_Helper;

	/**
	 * Holdes Class instance
	 *
	 * @var Automatic_Follower_Creator
	 */
	private static $instance = null;

	/**
	 * Automatic_Follower_Creator constructor.
	 */
	private function __construct() {
		$this->setup();
	}

	/**
	 * Boot class
	 */
	public static function boot() {

		if ( is_null( self::$instance ) ) {
			self::$instance = new self();
		}

		return self::$instance;
	}

	/**
	 * Setup actions
	 */
	private function setup() {
		add_action( 'bp_core_activated_user', array( $this, 'on_user_activate' ) );
		add_action( 'user_register', array( $this, 'on_registration' ) );
		add_action( 'xprofile_updated_profile', array( $this, 'on_profile_update' ) );
		add_action( 'set_user_role', array( $this, 'on_role_update' ) );
		add_action( 'add_user_role', array( $this, 'on_role_update' ) );

		add_action( 'bp_set_member_type', array( $this, 'on_member_type_update' ) );
		add_action( 'bp_remove_member_type', array( $this, 'on_member_type_update' ) );
	}

	/**
	 * On user activate
	 *
	 * @param int $user_id user id.
	 */
	public function on_user_activate( $user_id ) {

		if ( bpafp_is_enabled_auto_friendship_on_activation() ) {
			$this->add_followers( $user_id );
		}
	}

	/**
	 * Triggers matching on 'user_register' action.
	 *
	 * It only adds the global lists since xprofile data is not available.
	 *
	 * @param int $user_id user id.
	 */
	public function on_registration( $user_id ) {
		if ( bpafp_is_enabled_auto_friendship_on_registration() ) {
			$this->add_followers( $user_id );
		}
	}

	/**
	 * Runs on profile update.
	 *
	 * @param int $user_id User id.
	 */
	public function on_profile_update( $user_id ) {
		if ( bpafp_is_auto_friendship_enabled_on_profile_update() ) {
			$this->add_followers( $user_id );
		}
	}

	/**
	 * Runs follow on role update.
	 *
	 * @param int $user_id user id.
	 */
	public function on_role_update( $user_id ) {
		if ( bpafp_is_auto_friendship_enabled_on_role_change() ) {
			$this->add_followers( $user_id );
		}
	}

	/**
	 * Runs auto friendship on member type update.
	 *
	 * @param int $user_id user id.
	 */
	public function on_member_type_update( $user_id ) {
		if ( bpafp_is_auto_friendship_enabled_on_member_type_change() ) {
			$this->add_followers( $user_id );
		}
	}

	/**
	 * Add friends based on the matching lists.
	 *
	 * @param int $leader_id user id.
	 */
	public function add_followers( $leader_id ) {

		$matching_list_ids = bpafp_get_matching_friendship_list_ids( $leader_id, bpafp_get_active_lists( 'followers' ) );

		if ( ! $matching_list_ids ) {
			return;
		}

		foreach ( $matching_list_ids as $list_id ) {
			$this->create_user_followers( $leader_id, $list_id );
		}
	}

	/**
	 * Add followers form friendship lists
	 *
	 * @param int        $user_id User id.
	 * @param \WP_Post[] $lists Friendship List ids.
	 */
	public static function create_followers( $user_id, $lists ) {

		$matching_list_ids = bpafp_get_matching_friendship_list_ids( $user_id, $lists );

		if ( ! $matching_list_ids ) {
			return;
		}

		$self = self::boot();

		foreach ( $matching_list_ids as $list_id ) {
			$self->create_user_followers( $user_id, $list_id );
		}
	}

	/**
	 * Add list members as followers
	 *
	 * @param int $user_id User id.
	 * @param int $list_id List id.
	 */
	public static function add_list_members_as_followers( $user_id, $list_id ) {
		$self = self::boot();

		$self->create_user_followers( $user_id, $list_id );
	}

	/**
	 * Sync user follower list
	 *
	 * @param int  $user_id   User id.
	 * @param int  $list_id   List id.
	 * @param bool $hard_sync Hard sync enabled or not.
	 */
	public static function sync( $user_id, $list_id, $hard_sync = false ) {

		if ( ! $user_id || ! $list_id ) {
			return;
		}

		$self = self::boot();

		$list_members = bpafp_get_list_members( $list_id );

		if ( $hard_sync ) {
			$follower_ids           = $self->get_followers( $user_id );
			$removable_follower_ids = $follower_ids ? array_diff( $follower_ids, $list_members ) : array();

			foreach ( $removable_follower_ids as $follower_id ) {
				$self->remove_follower( $user_id, $follower_id );
			}
		}

		$self->create_user_followers( $user_id, $list_id );
	}

	/**
	 * Create friendship
	 *
	 * @param int $leader_id          User id.
	 * @param int $follower_list_id Friendship id.
	 */
	private function create_user_followers( $leader_id, $follower_list_id ) {
		$attached_user_ids = bpafp_get_list_members( $follower_list_id );

		if ( ! $attached_user_ids ) {
			return;
		}

		foreach ( $attached_user_ids as $user_id ) {
			$user = get_user_by( 'id', $user_id );

			if ( ! $user || $this->is_following( $user->ID, $leader_id ) ) {
				continue;
			}

			$this->add_follower( $leader_id, $user->ID );
		}
	}

	/**
	 * Adds user as to follower
	 *
	 * @param int $leader_id   Leader id.
	 * @param int $follower_id Follower id.
	 */
	public function start_following( $leader_id, $follower_id ) {

		if ( ! $this->is_following( $follower_id, $leader_id ) ) {
			$this->add_follower( $leader_id, $follower_id );
		}
	}
}