<?php
/**
 * Class to remove following automatic
 *
 * @package    BuddyPress_Auto_Friendship_Pro
 * @subpackage Core\Follow_Following
 * @copyright  Copyright (c) 2020, Brajesh Singh
 * @license    https://www.gnu.org/licenses/gpl.html GNU Public License
 * @author     Ravi Sharma, Brajesh Singh
 * @since      1.0.0
 */

namespace BuddyPress_Auto_Friendship_Pro\Core\Follow_Following;

use BuddyPress_Auto_Friendship_Pro\Core\User_List_Matcher;

// Exit if accessed directly.
defined( 'ABSPATH' ) || exit;

/**
 * Class Automatic_Following_Remover
 */
class Automatic_Following_Remover {

	use Follow_Helper;

	/**
	 * Holds class instance
	 *
	 * @var Automatic_Following_Remover
	 */
	private static $instance = null;

	/**
	 * Automatic_Following_Remover constructor.
	 */
	private function __construct() {
		$this->setup();
	}

	/**
	 * Returns class instance
	 *
	 * @return Automatic_Following_Remover
	 */
	public static function boot() {

		if ( is_null( self::$instance ) ) {
			self::$instance = new self();
		}

		return self::$instance;
	}

	/**
	 * Setup actions
	 */
	private function setup() {
		// Commented activation and registration call.
		// add_action( 'bp_core_activated_user', array( $this, 'on_user_activate' ) );
		// add_action( 'user_register', array( $this, 'on_registration' ) );
		add_action( 'xprofile_updated_profile', array( $this, 'on_profile_update' ) );
		add_action( 'set_user_role', array( $this, 'on_role_update' ) );
		add_action( 'add_user_role', array( $this, 'on_role_update' ) );

		add_action( 'bp_set_member_type', array( $this, 'on_member_type_update' ) );
		add_action( 'bp_remove_member_type', array( $this, 'on_member_type_update' ) );
	}

	/**
	 * On user activate
	 *
	 * @param int $user_id user id.
	 */
	public function on_user_activate( $user_id ) {}

	/**
	 * Triggers matching on 'user_register' action.
	 *
	 * It only adds the global lists since xprofile data is not available.
	 *
	 * @param int $user_id user id.
	 */
	public function on_registration( $user_id ) {}

	/**
	 * Runs on profile update.
	 *
	 * @param int $user_id User id.
	 */
	public function on_profile_update( $user_id ) {
		if ( bpafp_is_auto_friendship_enabled_on_profile_update() ) {
			$this->remove_user_following( $user_id );
		}
	}

	/**
	 * Runs follow on role update.
	 *
	 * @param int $user_id user id.
	 */
	public function on_role_update( $user_id ) {
		if ( bpafp_is_auto_friendship_enabled_on_role_change() ) {
			$this->remove_user_following( $user_id );
		}
	}

	/**
	 * Runs auto friendship on member type update.
	 *
	 * @param int $user_id user id.
	 */
	public function on_member_type_update( $user_id ) {
		if ( bpafp_is_auto_friendship_enabled_on_member_type_change() ) {
			$this->remove_user_following( $user_id );
		}
	}

	/**
	 * Remove user following based on the matching lists.
	 *
	 * @param int $follower_user_id user id.
	 */
	public function remove_user_following( $follower_user_id ) {
		$matcher = new User_List_Matcher( $follower_user_id );

		$following_list_ids = $matcher->get_matching_list_ids( bpafp_get_active_lists( 'unfollowing' ) );

		if ( ! $following_list_ids ) {
			return;
		}

		foreach ( $following_list_ids as $list_id ) {
			$this->delete_user_following( $follower_user_id, $list_id );
		}
	}

	/**
	 * Remove list members as following
	 *
	 * @param int $user_id User id.
	 * @param int $list_id List id.
	 */
	public static function remove_list_members_as_following( $user_id, $list_id ) {
		$self = self::boot();

		$self->delete_user_following( $user_id, $list_id );
	}

	/**
	 * Sync user unfollowing
	 *
	 * @param int  $user_id   User id.
	 * @param int  $list_id   List id.
	 * @param bool $harh_sync Hard sync enabled or not.
	 */
	public static function sync( $user_id, $list_id, $harh_sync = false ) {

		if ( ! $user_id || ! $list_id ) {
			return;
		}

		$self = self::boot();

		$self->delete_user_following( $user_id, $list_id );
	}

	/**
	 * Delete user following
	 *
	 * @param int $follower_user_id User id.
	 * @param int $following_list_id Friendship id.
	 */
	private function delete_user_following( $follower_user_id, $following_list_id ) {
		$attached_user_ids = bpafp_get_list_members( $following_list_id );

		if ( ! $attached_user_ids ) {
			return;
		}

		foreach ( $attached_user_ids as $user_id ) {
			$user = get_user_by( 'id', $user_id );

			if ( $user && $this->is_following( $follower_user_id, $user->ID ) ) {
				$this->remove_follower( $user->ID, $follower_user_id );
			}
		}
	}
}
