<?php
/**
 * Action handler class
 *
 * @package BuddyPress_Auto_Friendship_Pro
 * @subpackage Handlers
 */

namespace BuddyPress_Auto_Friendship_Pro\Handlers;

// Do not allow direct access over web.
use BuddyPress_Auto_Friendship_Pro\Core\Automatic_Friendship_Creator;
use BuddyPress_Auto_Friendship_Pro\Core\Follow_Following\Automatic_Follower_Creator;
use BuddyPress_Auto_Friendship_Pro\Core\Follow_Following\Automatic_Following_Creator;

defined( 'ABSPATH' ) || exit;

/**
 * Class Actions_Handler
 */
class Actions_Handler {

	/**
	 * Class self boot
	 */
	public static function boot() {
		$self = new self();
		$self->setup();
	}

	/**
	 * Setup
	 */
	private function setup() {
		add_action( 'wp_login', array( $this, 'on_login' ), -10, 2 );
		add_action( 'bp_invitations_accepted_request', array( $this, 'on_accept' ) );
		add_action( 'members_invitations_invite_accepted', array( $this, 'on_invite_accept' ), 10, 3 );
		add_action( 'bp_invites_member_invite_mark_register_user', array( $this, 'on_bb_accept' ), 10, 2 );
		add_action( 'bp_invites_member_invite_activate_user', array( $this, 'on_bb_accept' ), 10, 2 );
	}

	/**
	 * On login create friendship follower etc.
	 *
	 * @param string   $user_login Username.
	 * @param \WP_User $user       WP_User object of the logged-in user.
	 */
	public function on_login( $user_login, $user ) {
		$triggers = (array) bpafp_get_option( 'auto_friendship_triggers', array() );

		if ( in_array( 'first_login', $triggers, true ) && empty( bp_get_user_last_activity( $user->ID ) ) ) {

			if ( function_exists( 'friends_check_friendship' ) ) {
				$this->create_user_friendships( $user->ID );
			}

			if ( bpafp_is_active_follow_following() ) {
				$this->create_user_follow_followings( $user->ID );
			}
		}
	}

	/**
	 * Create user friendships
	 *
	 * @param int $user_id User id.
	 */
	private function create_user_friendships( $user_id ) {
		$lists = bpafp_get_active_lists( 'friends' );

		if ( ! $lists ) {
			return;
		}

		Automatic_Friendship_Creator::create_connections( $user_id, $lists );
	}

	/**
	 * Create user follow following
	 *
	 * @param int $user_id User id.
	 */
	private function create_user_follow_followings( $user_id ) {
		$follower_lists  = bpafp_get_active_lists( 'followers' );
		$following_lists = bpafp_get_active_lists( 'followings' );

		if ( $follower_lists ) {
			Automatic_Follower_Creator::create_followers( $user_id, $follower_lists );
		}

		if ( $following_lists ) {
			Automatic_Following_Creator::create_following( $user_id, $following_lists );
		}
	}

	/**
	 * Process after invitation accepted.
	 *
	 * @param array $r invitation args.
	 */
	public function on_accept( $r ) {

		if ( ! $r['user_id'] || ! $r['inviter_id'] ) {
			return;
		}

		$settings = array_filter( (array) bpafp_get_option( 'invited_connection_types', array() ) );

		if ( empty( $settings ) ) {
			return;
		}

		if ( in_array( 'friend', $settings ) && function_exists( 'friends_check_friendship' ) && ! friends_check_friendship( $r['user_id'], $r['inviter_id'] ) ) {
			friends_add_friend( $r['user_id'], $r['inviter_id'], true );
		}

		if ( in_array( 'follower', $settings ) ) {
			Automatic_Follower_Creator::boot()->start_following( $r['inviter_id'], $r['user_id'] );
		}

		if ( in_array( 'following', $settings ) ) {
			Automatic_Follower_Creator::boot()->start_following( $r['user_id'], $r['inviter_id'] );
		}
	}

	/**
	 * Runs on invite accepted success.
	 *
	 * @param \BP_Invitation $invite Invite object.
	 * @param \WP_User       $invitee_user User who accepted the request.
	 * @param int            $inviter_id User id who invited the user.
	 */
	public function on_invite_accept( $invite, $invitee_user, $inviter_id ) {

		if ( $inviter_id && ! empty( $invitee_user->ID ) ) {
			$args = array(
				'user_id'    => $invitee_user->ID,
				'inviter_id' => $inviter_id,
			);

			$this->on_accept( $args );
		}
	}

	/**
	 * On BuddyBoss invite register
	 *
	 * @param int $user_id User id.
	 * @param int $inviter_id Inviter id.
	 */
	public function on_bb_accept( $user_id, $inviter_id ) {

		$args = array(
			'user_id'    => $user_id,
			'inviter_id' => $inviter_id,
		);

		$this->on_accept( $args );
	}
}
