<?php
/**
 * Action handler class
 *
 * @package    BuddyPress_Auto_Friendship_Pro
 * @subpackage Handlers
 */

namespace BuddyPress_Auto_Friendship_Pro\Handlers;

use BuddyPress_Auto_Friendship_Pro\Core\Automatic_Friendship_Creator;
use BuddyPress_Auto_Friendship_Pro\Core\Automatic_Friendship_Remover;
use BuddyPress_Auto_Friendship_Pro\Core\Follow_Following\Automatic_Following_Creator;
use BuddyPress_Auto_Friendship_Pro\Core\Follow_Following\Automatic_Following_Remover;
use BuddyPress_Auto_Friendship_Pro\Core\Follow_Following\Automatic_Follower_Creator;
use BuddyPress_Auto_Friendship_Pro\Core\Follow_Following\Automatic_Follower_Remover;
use BuddyPress_Auto_Friendship_Pro\Core\User_List_Matcher;

// Do not allow direct access over web.
defined( 'ABSPATH' ) || exit;

/**
 * Class Ajax_Handler
 */
class Ajax_Handler {

	/**
	 * Class self boot
	 */
	public static function boot() {
		$self = new self();
		$self->setup();
	}

	/**
	 * Setup
	 */
	private function setup() {
		add_action( 'wp_ajax_bpafp_get_members', array( $this, 'get_members' ) );
		add_action( 'wp_ajax_bpafp_sync_friends', array( $this, 'sync_friends' ) );
	}

	/**
	 * Get members
	 */
	public function get_members() {
		check_ajax_referer( 'bpafp-get-members' );

		$search_query = isset( $_POST['q'] ) ? sanitize_text_field( wp_unslash( $_POST['q'] ) ) : '';

		if ( empty( $search_query ) ) {
			wp_send_json_error( __( 'Please type some letters for suggestion.', 'buddypress-auto-friendship-pro' ) );
		}

		$query_args = array(
			'search_terms' => $search_query,
			'type'         => 'alphabetical', // To get all users.
		);

		$members = bp_core_get_users( $query_args );

		if ( empty( $members['total'] ) ) {
			wp_send_json_error( array() ); //__( 'No member found', 'buddypress-auto-friendship-pro' )
		}

		$list = array();

		foreach ( $members['users'] as $user ) {
			$list[] = array(
				'label' => $user->display_name,
				'icon'  => bp_core_fetch_avatar( array(
					'item_id' => $user->ID,
					'width'   => 60,
					'height'  => 60,
				) ),
				'url'   => bpafp_get_user_url( $user->ID ),
				'id'    => $user->ID,
			);
		}

		wp_send_json_success( $list );
	}

	/**
	 * Sync Friends
	 */
	public function sync_friends() {
		check_ajax_referer( 'bpafp-sync-friends' );

		if ( ! is_super_admin() ) {
			wp_send_json_error( __( 'Invalid action.', 'buddypress-auto-friendship-pro' ) );
		}

		$list      = empty( $_POST['list_id'] ) ? 0 : get_post( absint( $_POST['list_id'] ) );
		$hard_sync = empty( $_POST['hard_sync'] ) ? 0 : 1;

		if ( ! $list || 'publish' != $list->post_status || $list->post_type != bpafp_get_list_post_type() ) { // should we further validate list for type etc?
			wp_send_json_error( __( 'Invalid list.', 'buddypress-auto-friendship-pro' ) );
		}

		global $wpdb;
		$total_users = $wpdb->get_var( "SELECT COUNT(ID) FROM {$wpdb->users}" );

		$list_type = get_post_meta( $list->ID, '_bpafp_list_type', true );

		$skipped           = absint( get_post_meta( $list->ID, '_bpafp_skipped', true ) );
		$processed         = absint( get_post_meta( $list->ID, '_bpafp_processed', true ) );
		$last_processed_id = absint( get_post_meta( $list->ID, '_bpafp_last_processed_user', true ) );

		$next_remaining_users = $this->get_remaining_user_ids( $last_processed_id );

		// if no more left.
		if ( empty( $next_remaining_users ) ) {
			$this->clear_state( $list->ID );

			wp_send_json_success(
				array(
					'total'     => $total_users,
					'remaining' => 0,
					'skipped'   => $skipped,
					'processed' => $processed,
				)
			);
		}

		foreach ( $next_remaining_users as $user_id ) {
			$is_match          = User_List_Matcher::check_if_list_matches( $user_id, $list );
			$last_processed_id = $user_id;

			if ( ! $is_match ) {
				$skipped ++;
				continue;
			}

			if ( 'followers' === $list_type ) {
				Automatic_Follower_Creator::sync( $user_id, $list->ID, $hard_sync );
			} elseif ( 'followings' === $list_type ) {
				Automatic_Following_Creator::sync( $user_id, $list->ID, $hard_sync );
			} elseif ( 'friends' === $list_type ) {
				Automatic_Friendship_Creator::sync( $user_id, $list->ID, $hard_sync );
			} elseif ( 'unfriend' === $list_type ) {
				Automatic_Friendship_Remover::sync( $user_id, $list->ID, $hard_sync );
			} elseif ( 'unfollow' === $list_type ) {
				Automatic_Follower_Remover::sync( $user_id, $list->ID, $hard_sync );
			} elseif ( 'unfollowing' === $list_type ) {
				Automatic_Following_Remover::sync( $user_id, $list->ID, $hard_sync );
			}

			$processed ++;
		}

		// save current state.
		update_post_meta( $list->ID, '_bpafp_last_processed_user', $last_processed_id );
		update_post_meta( $list->ID, '_bpafp_skipped', $skipped );
		update_post_meta( $list->ID, '_bpafp_processed', $processed );

		$remaining = $total_users - $skipped - $processed;

		if ( empty( $remaining ) ) {
			$this->clear_state( $list->ID );
		}

		wp_send_json_success(
			array(
				'total'     => $total_users,
				'remaining' => $remaining,
				'skipped'   => $skipped,
				'processed' => $processed,
			)
		);
	}

	/**
	 * Create state
	 *
	 * @param int $list_id List id.
	 */
	private function clear_state( $list_id ) {
		delete_post_meta( $list_id, '_bpafp_skipped' );
		delete_post_meta( $list_id, '_bpafp_processed' );
		delete_post_meta( $list_id, '_bpafp_last_processed_user' );
	}

	/**
	 * Get remaining users
	 *
	 * @param int $last_processed_user_id last processed user id.
	 *
	 * @return array
	 */
	private function get_remaining_user_ids( $last_processed_user_id ) {
		global $wpdb;

		$query = $wpdb->prepare( "SELECT ID FROM {$wpdb->users} WHERE ID > %d ORDER BY ID ASC LIMIT %d", absint( $last_processed_user_id ), 10 );

		return $wpdb->get_col( $query );
	}
}

