<?php
/**
 * Group Admin Settings Pages Helper.
 *
 * @package    BuddyPress_Auto_Friendship_Pro
 * @subpackage Modules\Groups
 * @copyright  Copyright (c) 2022, Brajesh Singh
 * @license    https://www.gnu.org/licenses/gpl.html GNU Public License
 * @author     Ravi Sharma, Brajesh Singh
 * @since      1.0.0
 */

namespace BuddyPress_Auto_Friendship_Pro\Modules\Groups;

use BP_Groups_Member;
use BuddyPress_Auto_Friendship_Pro\Core\Follow_Following\Automatic_Follower_Creator;
use BuddyPress_Auto_Friendship_Pro\Core\Follow_Following\Automatic_Follower_Remover;

// Do not allow direct access over web.
defined( 'ABSPATH' ) || exit;

/**
 * Class handle group actions
 */
class Group_Actions_Handler {

	/**
	 * Boot settings
	 */
	public static function boot() {
		$self = new self();

		$self->setup();
	}

	/**
	 * Setup settings
	 */
	public function setup() {
		add_action( 'groups_join_group', array( $this, 'on_join' ), 10, 2 );
		add_action( 'groups_leave_group', array( $this, 'on_leave' ), 10, 2 );
		//add_action( 'groups_removed_member', array( $this, 'on_remove' ), 10, 2 );
		add_action( 'groups_remove_member', array( $this, 'on_remove' ), 10, 2 );
		add_action( 'bp_rest_group_members_delete_item', array( $this, 'on_rest_remove' ), 10, 5 );
	}

	/**
	 * Performs relationship building on group join as per settings.
	 *
	 * @param int $group_id Group id.
	 * @param int $user_id  ID of the user joining the group.
	 */
	public function on_join( $group_id, $user_id ) {
		$connect_with     = (array) bpafp_get_option( 'group_join_connect_with', array() );
		$connection_types = (array) bpafp_get_option( 'group_join_connection_types', array() );

		if ( ! $connect_with || ! $connection_types ) {
			return;
		}

		$member_ids = $this->get_group_member_ids( $group_id, $connect_with );

		if ( ! $member_ids ) {
			return;
		}

		foreach ( $connection_types as $connection_type ) {
			$this->connect( $user_id, $member_ids, $connection_type );
		}
	}

	/**
	 * Performs relationship canceling on group leave as per settings.
	 *
	 * @param int $group_id Group id.
	 * @param int $user_id ID of the user joining the group.
	 */
	public function on_leave( $group_id, $user_id ) {
		$connect_with     = (array) bpafp_get_option( 'group_leave_connect_with', array() );
		$connection_types = (array) bpafp_get_option( 'group_leave_connection_types', array() );

		if ( ! $connect_with || ! $connection_types ) {
			return;
		}

		$member_ids = $this->get_group_member_ids( $group_id, $connect_with );

		if ( ! $member_ids ) {
			return;
		}

		foreach ( $connection_types as $connection_type ) {
			$this->disconnect( $user_id, $member_ids, $connection_type );
		}
	}

	/**
	 * Performs relationship canceling on group removal as per settings.
	 *
	 * @param int $group_id ID of the group the user is removed from.
	 * @param int $user_id  ID of the user being removed.
	 */
	public function on_remove(  $group_id, $user_id ) {
		$connect_with     = (array) bpafp_get_option( 'group_remove_connect_with', array() );
		$connection_types = (array) bpafp_get_option( 'group_remove_connection_types', array() );

		if ( ! $connect_with || ! $connection_types ) {
			return;
		}

		$member_ids = $this->get_group_member_ids( $group_id, $connect_with );

		if ( ! $member_ids ) {
			return;
		}

		foreach ( $connection_types as $connection_type ) {
			$this->disconnect( $user_id, $member_ids, $connection_type );
		}
	}

	/**
	 * Runs on rest delete.
	 *
	 * @param \WP_User          $user     The updated member.
	 * @param \BP_Groups_Member $member   The group member object.
	 * @param \BP_Groups_Group  $group    The group object.
	 * @param \WP_REST_Response $response The response data.
	 * @param \WP_REST_Request  $request  The request sent to the API.
	 */
	public function on_rest_remove( $user, $member, $group, $response, $request ) {
		$group_id = $request->get_param( 'group_id' );
		$user_id  = $request->get_param( 'user_id' );

		$connect_with     = (array) bpafp_get_option( 'group_remove_connect_with', array() );
		$connection_types = (array) bpafp_get_option( 'group_remove_connection_types', array() );

		if ( ! $connect_with || ! $connection_types ) {
			return;
		}

		$member_ids = $this->get_group_member_ids( $group_id, $connect_with );

		if ( ! $member_ids ) {
			return;
		}

		foreach ( $connection_types as $connection_type ) {
			$this->disconnect( $user_id, $member_ids, $connection_type );
		}
	}

	/**
	 * Performs building relationship based on type
	 *
	 * @param int    $user_id       User id.
	 * @param int[]  $member_ids    Member ids.
	 * @param string $connection_type Relationship type.
	 */
	private function connect( $user_id, $member_ids, $connection_type ) {
		$connection_types = bpafp_get_available_connection_type_options();

		if ( ! array_key_exists( $connection_type, $connection_types ) ) {
			return;
		}

		foreach ( $member_ids as $member_id ) {
			//$member = get_user_by( 'id', $member_id );

			//if ( ! $member ) {
				// continue;
			//}

			if ( 'friend' == $connection_type
			     && function_exists( 'friends_check_friendship')
			     && ! friends_check_friendship( $user_id, $member_id )
			) {
				friends_add_friend( $member_id, $user_id, true );
			}

			$is_active_follow = bpafp_is_active_follow_following();

			if ( $is_active_follow && 'follower' == $connection_type ) {
				Automatic_Follower_Creator::boot()->start_following( $member_id, $user_id );
			}

			if ( $is_active_follow && 'following' == $connection_type ) {
				Automatic_Follower_Creator::boot()->start_following( $user_id, $member_id );
			}
		}
	}

	/**
	 * Performs canceling relationship based on type
	 *
	 * @param int    $user_id       User id.
	 * @param int[]  $member_ids    Member ids.
	 * @param string $connection_type Relationship type.
	 */
	private function disconnect( $user_id, $member_ids, $connection_type ) {
		$connection_types = bpafp_get_available_connection_type_options();

		if ( ! array_key_exists( $connection_type, $connection_types ) ) {
			return;
		}

		foreach ( $member_ids as $member_id ) {
			//$member = get_user_by( 'id', $member_id );

			//if ( ! $member ) {
			//	continue;
			//}

			if ( 'friend' == $connection_type
			     && function_exists( 'friends_check_friendship' )
			     && friends_check_friendship( $user_id, $member_id )
			) {
				friends_remove_friend( $user_id, $member_id );
			}

			$is_follow_active = bpafp_is_active_follow_following();

			if ( $is_follow_active && 'follower' == $connection_type ) {
				Automatic_Follower_Remover::boot()->stop_following( $member_id, $user_id );
			}

			if ( $is_follow_active && 'following' == $connection_type ) {
				Automatic_Follower_Remover::boot()->stop_following( $user_id, $member_id );
			}
		}
	}

	/**
	 * Get group member ids
	 *
	 * @param int   $group_id     Members need to find.
	 * @param array $connect_with Connect with option.
	 *
	 * @return int[]
	 */
	private function get_group_member_ids( $group_id, $connect_with ) {

		if ( ! $connect_with ) {
			return array();
		}

		$all_member_ids = BP_Groups_Member::get_group_member_ids( $group_id );

		/*groups_get_group_members(
			array(
				'group_id'            => $group_id,
				'per_page'            => false,
				'exclude_admins_mods' => 0,
			)
		);*/

		if ( in_array( 'all', $connect_with ) ) {
			return $all_member_ids;
		}

		$user_ids  = array();

		$admins    = groups_get_group_admins( $group_id );
		$admin_ids = $admins ? wp_list_pluck( $admins, 'user_id' ) : array();

		$mods    = groups_get_group_mods( $group_id );
		$mod_ids = $mods ? wp_list_pluck( $mods, 'user_id' ) : array();

		if ( in_array( 'admins', $connect_with ) ) {
			$user_ids = array_merge( $user_ids, $admin_ids );
		}

		if ( in_array( 'moderators', $connect_with ) ) {
			$user_ids = array_merge( $user_ids, $mod_ids );
		}

		if ( in_array( 'members', $connect_with ) ) {
			$member_ids = array_diff( $all_member_ids, $admin_ids, $mod_ids );
			$user_ids   = array_merge( $user_ids, $member_ids );
		}

		return array_unique( $user_ids );
	}
}
